// ========================================================
//
// CBP_2403_D_ObjAvoid_FSM 
// FSM for USonic Obj Avoid.
// CBP 11-09-21 
// Status Devel
//
// ========================================================

#include <stdio.h>
#include <Servo.h>

// Define pins ---------------
#define HC_SR04_Tx 2
#define HC_SR04_Rx 3
#define ServoL_pin 13
#define ServoR_pin 12

// Define constants ----------
#define dT 20
#define DEBUG true
#define wheelDiam 66.0
#define wheelRad (wheelDiam/2.0)
#define axleLen 104.0

// Define states
#define STATE_FORWARD 1
#define STATE_BACKWARD 2
#define STATE_TURN_LEFT 3

// Global variables ----------
Servo servoL;
Servo servoR; 
float driveL, driveR;
float dist;
int state;
void setup() {
  Serial.begin(9600);
  cprintf(">>> Initializing stuff ..................\n");
  pinMode(HC_SR04_Tx,OUTPUT);
  pinMode(HC_SR04_Rx,INPUT);
  servoL.attach(ServoL_pin);
  servoR.attach(ServoR_pin);
  // Stop any servo movement
  driveServos(0.0,0.0);  
  
  cprintf(">>> Pausing 2 secs ......................\n");
  delay(2000);
  cprintf(">>> Starting  ...........................\n");

  state = STATE_FORWARD;
}


void loop() {

  switch(state) {
    case STATE_FORWARD :
      driveL = 30;
      driveR = 30;
      driveServos(driveL,driveR);
      dist = getDistance();
      delay(60); // Required by HC_SR04 datasheet
      cprintf("Dist = %f\n",dist);
      if ( (dist < 300) && (dist != -1) ) {
        cprintf("Transit to state STATE_BACKWARD\n");
        state = STATE_BACKWARD;
      }
      break;

    case STATE_BACKWARD :
      driveL = -30;
      driveR = -30;
      driveServos(driveL,driveR);
      delay(3000);
      cprintf("Transit to state STATE_TURN_LEFT\n"); 
      state = STATE_TURN_LEFT;
      break;
    case STATE_TURN_LEFT :
      driveL = 30;
      driveR = -30;
      driveServos(driveL,driveR);
      delay(3000);
      cprintf("Transit to state STATE_FORWARD\n"); 
      state = STATE_FORWARD;
      break;         
  }

  
}

float getDistance() {
  long duration;
  float mm;
  digitalWrite(HC_SR04_Tx, LOW);
  delayMicroseconds(2);
  digitalWrite(HC_SR04_Tx, HIGH);
  delayMicroseconds(10);
  digitalWrite(HC_SR04_Tx, LOW);

  duration = pulseIn(HC_SR04_Rx, HIGH);
  mm = 10*duration / 29 / 2;

  // Catch 'bad' values and report as mm = -1;
  if(duration < 600) mm = -1;
  if(mm > 4000) mm = -1;  
  //if(DEBUG) cprintf("Dist (mm) = %f\n",mm);
  return mm;
  
}

void driveServos(float _driveL, float _driveR) {
  servoL.writeMicroseconds(1500 + (int)_driveL);
  servoR.writeMicroseconds(1500 - (int)_driveR);
}
